# Christian Baker, King's College London
# Copyright (c) 2022 School of Biomedical Engineering & Imaging Sciences, King's College London
# Licensed under the MIT. You may obtain a copy at https://opensource.org/licenses/MIT.

""" pymagewell provides a high-level python interface to the Magewell ProCapture SDK """
