# Christian Baker, King's College London
# Copyright (c) 2022 School of Biomedical Engineering & Imaging Sciences, King's College London
# Licensed under the MIT. You may obtain a copy at https://opensource.org/licenses/MIT.

from pymagewell.pro_capture_device.pro_capture_device import ProCaptureDevice

__all__ = ["ProCaptureDevice"]
